@@ -22,6 +22,7 @@ import contextlib
 import pathlib
 import time
 import typing
+from simvue.api.objects.alert.user import UserAlert
 
 if typing.TYPE_CHECKING:
     import simvue
@@ -35,13 +36,13 @@ class CompletionCallback(typing.Protocol):
 
 def _execute_process(
     proc_id: str,
-    command: typing.List[str],
+    command: list[str],
     runner_name: str,
-    completion_callback: typing.Optional[CompletionCallback] = None,
-    completion_trigger: typing.Optional[multiprocessing.synchronize.Event] = None,
-    environment: typing.Optional[typing.Dict[str, str]] = None,
-    cwd: typing.Optional[pathlib.Path] = None,
-) -> tuple[subprocess.Popen, typing.Optional[threading.Thread]]:
+    completion_callback: CompletionCallback | None = None,
+    completion_trigger: multiprocessing.synchronize.Event | None = None,
+    environment: dict[str, str] | None = None,
+    cwd: pathlib.Path | None = None,
+) -> tuple[subprocess.Popen, threading.Thread | None]:
     thread_out = None
 
     with open(f"{runner_name}_{proc_id}.err", "w") as err:
@@ -58,8 +59,8 @@ def _execute_process(
     if completion_callback or completion_trigger:
 
         def trigger_check(
-            completion_callback: typing.Optional[CompletionCallback],
-            trigger_to_set: typing.Optional[multiprocessing.synchronize.Event],
+            completion_callback: CompletionCallback | None,
+            trigger_to_set: multiprocessing.synchronize.Event | None,
             process: subprocess.Popen,
         ) -> None:
             while process.poll() is None:
@@ -67,12 +68,8 @@ def _execute_process(
             if trigger_to_set:
                 trigger_to_set.set()
             if completion_callback:
-                with open(f"{runner_name}_{proc_id}.err") as err:
-                    std_err = err.read()
-
-                with open(f"{runner_name}_{proc_id}.out") as out:
-                    std_out = out.read()
-
+                std_err = pathlib.Path(f"{runner_name}_{proc_id}.err").read_text()
+                std_out = pathlib.Path(f"{runner_name}_{proc_id}.out").read_text()
                 completion_callback(
                     status_code=process.returncode,
                     std_out=std_out,
@@ -109,23 +106,24 @@ class Executor:
         """
         self._runner = simvue_runner
         self._keep_logs = keep_logs
-        self._completion_callbacks: dict[str, typing.Optional[CompletionCallback]] = {}
+        self._completion_callbacks: dict[str, CompletionCallback] | None = {}
         self._completion_triggers: dict[
-            str, typing.Optional[multiprocessing.synchronize.Event]
+            str, multiprocessing.synchronize.Event | None
         ] = {}
-        self._completion_processes: dict[str, typing.Optional[threading.Thread]] = {}
+        self._completion_processes: dict[str, threading.Thread] | None = {}
         self._alert_ids: dict[str, str] = {}
         self.command_str: dict[str, str] = {}
         self._processes: dict[str, subprocess.Popen] = {}
+        self._all_processes: list[psutil.Process] = []
 
-    def std_out(self, process_id: str) -> typing.Optional[str]:
+    def std_out(self, process_id: str) -> str | None:
         if not os.path.exists(out_file := f"{self._runner.name}_{process_id}.out"):
             return None
 
         with open(out_file) as out:
             return out.read() or None
 
-    def std_err(self, process_id: str) -> typing.Optional[str]:
+    def std_err(self, process_id: str) -> str | None:
         if not os.path.exists(err_file := f"{self._runner.name}_{process_id}.err"):
             return None
 
@@ -136,13 +134,13 @@ class Executor:
         self,
         identifier: str,
         *args,
-        executable: typing.Optional[str] = None,
-        script: typing.Optional[pathlib.Path] = None,
-        input_file: typing.Optional[pathlib.Path] = None,
-        env: typing.Optional[typing.Dict[str, str]] = None,
-        cwd: typing.Optional[pathlib.Path] = None,
-        completion_callback: typing.Optional[CompletionCallback] = None,
-        completion_trigger: typing.Optional[multiprocessing.synchronize.Event] = None,
+        executable: str | None = None,
+        script: pathlib.Path | None = None,
+        input_file: pathlib.Path | None = None,
+        env: dict[str, str] | None = None,
+        cwd: pathlib.Path | None = None,
+        completion_callback: CompletionCallback | None = None,
+        completion_trigger: multiprocessing.synchronize.Event | None = None,
         **kwargs,
     ) -> None:
         """Add a process to be executed to the executor.
@@ -187,9 +185,9 @@ class Executor:
         input_file : str | None, optional
             the input file to run, note this only work if the input file is not an option, if this is the case
             you should provide it as such and perform the upload manually, by default None
-        env : typing.Dict[str, str], optional
+        env : dict[str, str], optional
             environment variables for process
-        cwd: typing.Optional[pathlib.Path], optional
+        cwd: pathlib.Path | None, optional
             working directory to execute the process within
         completion_callback : typing.Callable | None, optional
             callback to run when process terminates
@@ -213,7 +211,7 @@ class Executor:
         if input_file:
             self._runner.save_file(file_path=input_file, category="input")
 
-        command: typing.List[str] = []
+        command: list[str] = []
 
         if executable:
             command += [f"{executable}"]
@@ -233,15 +231,15 @@ class Executor:
             arg = arg.replace("_", "-")
 
             if len(arg) == 1:
-                if isinstance(value, bool) and value:
-                    command += [f"-{arg}"]
-                else:
-                    command += [f"-{arg}", f"{value}"]
+                command += (
+                    [f"-{arg}"]
+                    if isinstance(value, bool) and value
+                    else [f"-{arg}", f"{value}"]
+                )
+            elif isinstance(value, bool) and value:
+                command += [f"--{arg}"]
             else:
-                if isinstance(value, bool) and value:
-                    command += [f"--{arg}"]
-                else:
-                    command += [f"--{arg}", f"{value}"]
+                command += [f"--{arg}", f"{value}"]
 
         command += pos_args
 
@@ -261,8 +259,8 @@ class Executor:
             )
         )
 
-        self._alert_ids[identifier] = self._runner.create_alert(
-            name=f"{identifier}_exit_status", source="user"
+        self._alert_ids[identifier] = self._runner.create_user_alert(
+            name=f"{identifier}_exit_status"
         )
 
         if not self._alert_ids[identifier]:
@@ -274,19 +272,48 @@ class Executor:
         if not self._processes:
             return []
 
-        _all_processes: list[psutil.Process] = []
+        _current_processes: list[psutil.Process] = []
 
         for process in self._processes.values():
             with contextlib.suppress(psutil.NoSuchProcess):
-                _all_processes.append(psutil.Process(process.pid))
+                _current_processes.append(psutil.Process(process.pid))
 
         with contextlib.suppress(psutil.NoSuchProcess, psutil.ZombieProcess):
-            for process in _all_processes:
+            for process in _current_processes:
                 for child in process.children(recursive=True):
-                    if child not in _all_processes:
-                        _all_processes.append(child)
+                    if child not in _current_processes:
+                        _current_processes.append(child)
 
-        return list(set(_all_processes))
+        _current_pids = set([_process.pid for _process in _current_processes])
+        _previous_pids = set([_process.pid for _process in self._all_processes])
+
+        # Find processes which used to exist, which are no longer running
+        _expired_process_pids = _previous_pids - _current_pids
+
+        # Remove these processes from list of all processes
+        self._all_processes = [
+            _process
+            for _process in self._all_processes
+            if _process.pid not in _expired_process_pids
+        ]
+
+        # Find new processes
+        _new_process_pids = _current_pids - _previous_pids
+        _new_processes = [
+            _process
+            for _process in _current_processes
+            if _process.pid in _new_process_pids
+        ]
+
+        # Get CPU usage stats for each of those new processes, so that next time it's measured by the heartbeat the value is accurate
+        if _new_processes:
+            [_process.cpu_percent() for _process in _new_processes]
+            time.sleep(0.1)
+
+        # Add these to the list of all processes
+        self._all_processes += _new_processes
+
+        return self._all_processes
 
     @property
     def success(self) -> int:
@@ -296,16 +323,14 @@ class Executor:
     @property
     def exit_status(self) -> int:
         """Returns the first non-zero exit status if applicable"""
-        _non_zero = [
+        if _non_zero := [
             i.returncode for i in self._processes.values() if i.returncode != 0
-        ]
-
-        if _non_zero:
+        ]:
             return _non_zero[0]
 
         return 0
 
-    def get_error_summary(self) -> dict[str, typing.Optional[str]]:
+    def get_error_summary(self) -> dict[str, str] | None:
         """Returns the summary messages of all errors"""
         return {
             identifier: self._get_error_status(identifier)
@@ -330,8 +355,8 @@ class Executor:
             raise KeyError(f"Failed to retrieve '{process_id}', no such process")
         return self.command_str[process_id]
 
-    def _get_error_status(self, process_id: str) -> typing.Optional[str]:
-        err_msg: typing.Optional[str] = None
+    def _get_error_status(self, process_id: str) -> str | None:
+        err_msg: str | None = None
 
         # Return last 10 lines of stdout if stderr empty
         if not (err_msg := self.std_err(process_id)) and (
@@ -344,20 +369,31 @@ class Executor:
 
     def _update_alerts(self) -> None:
         """Send log events for the result of each process"""
+        # Wait for the dispatcher to send the latest information before
+        # allowing the executor to finish (and as such the run instance to exit)
+        _wait_limit: float = 1
         for proc_id, process in self._processes.items():
+            # We don't want to override the user's setting for the alert status
+            # This is so that if a process incorrectly reports its return code,
+            # the user can manually set the correct status depending on logs etc.
+            _alert = UserAlert(identifier=self._alert_ids[proc_id])
+            _is_set = _alert.get_status(run_id=self._runner._id)
+
             if process.returncode != 0:
                 # If the process fails then purge the dispatcher event queue
                 # and ensure that the stderr event is sent before the run closes
                 if self._runner._dispatcher:
                     self._runner._dispatcher.purge()
-
-                self._runner.log_alert(self._alert_ids[proc_id], "critical")
+                if not _is_set:
+                    self._runner.log_alert(
+                        identifier=self._alert_ids[proc_id], state="critical"
+                    )
             else:
-                self._runner.log_alert(self._alert_ids[proc_id], "ok")
+                if not _is_set:
+                    self._runner.log_alert(
+                        identifier=self._alert_ids[proc_id], state="ok"
+                    )
 
-            # Wait for the dispatcher to send the latest information before
-            # allowing the executor to finish (and as such the run instance to exit)
-            _wait_limit: float = 1
             _current_time: float = 0
             while (
                 self._runner._dispatcher
@@ -380,7 +416,7 @@ class Executor:
                 )
 
     def kill_process(
-        self, process_id: typing.Union[int, str], kill_children_only: bool = False
+        self, process_id: int | str, kill_children_only: bool = False
     ) -> None:
         """Kill a running process by ID
 
@@ -389,7 +425,7 @@ class Executor:
 
         Parameters
         ----------
-        process_id : typing.Union[int, str]
+        process_id : int | str
             either the identifier for a client created process or the PID
             of an external process
         kill_children_only : bool, optional
