@@ -16,14 +16,16 @@ from .remote import Remote
 
 
 def Simvue(
-    name: typing.Optional[str],
+    name: str | None,
     uniq_id: str,
     mode: str,
     config: "SimvueConfiguration",
     suppress_errors: bool = True,
 ) -> "SimvueBaseClass":
     if mode == "offline":
-        return Offline(name=name, uniq_id=uniq_id, suppress_errors=suppress_errors)
+        return Offline(
+            name=name, uniq_id=uniq_id, suppress_errors=suppress_errors, config=config
+        )
     else:
         return Remote(
             name=name, uniq_id=uniq_id, config=config, suppress_errors=suppress_errors
