@@ -2,6 +2,7 @@ import datetime
 import hashlib
 import logging
 import json
+import mimetypes
 import tabulate
 import pydantic
 import importlib.util
@@ -10,10 +11,11 @@ import contextlib
 import os
 import pathlib
 import typing
-
 import jwt
+from deepmerge import Merger
 
 from datetime import timezone
+from simvue.models import DATETIME_FORMAT
 
 
 CHECKSUM_BLOCK_SIZE = 4096
@@ -26,8 +28,8 @@ if typing.TYPE_CHECKING:
 
 
 def find_first_instance_of_file(
-    file_names: typing.Union[list[str], str], check_user_space: bool = True
-) -> typing.Optional[pathlib.Path]:
+    file_names: list[str] | str, check_user_space: bool = True
+) -> pathlib.Path | None:
     """Traverses a file hierarchy from bottom upwards to find file
 
     Returns the first instance of 'file_names' found when moving
@@ -35,7 +37,6 @@ def find_first_instance_of_file(
 
     Parameters
     ----------
-    file_name: list[str] | str
         candidate names of file to locate
     check_user_space: bool, optional
         check the users home area if current working directory is not
@@ -49,16 +50,17 @@ def find_first_instance_of_file(
     if isinstance(file_names, str):
         file_names = [file_names]
 
-    for root, _, files in os.walk(os.getcwd(), topdown=False):
-        for file_name in file_names:
-            if file_name in files:
-                return pathlib.Path(root).joinpath(file_name)
+    for file_name in file_names:
+        _user_file = pathlib.Path.cwd().joinpath(file_name)
+        if _user_file.exists():
+            return _user_file
 
     # If the user is running on different mounted volume or outside
     # of their user space then the above will not return the file
     if check_user_space:
         for file_name in file_names:
-            if os.path.exists(_user_file := pathlib.Path.home().joinpath(file_name)):
+            _user_file = pathlib.Path.home().joinpath(file_name)
+            if _user_file.exists():
                 return _user_file
 
     return None
@@ -90,16 +92,21 @@ def parse_validation_response(
 
     out: list[list[str]] = []
 
+    if isinstance(issues, str):
+        return tabulate.tabulate(
+            ["Unknown", "N/A", issues],
+            headers=["Type", "Location", "Message"],
+            tablefmt="fancy_grid",
+        )
+
     for issue in issues:
         obj_type: str = issue["type"]
         location: list[str] = issue["loc"]
         location.remove("body")
-        location_addr: str = ""
-        for i, loc in enumerate(location):
-            if isinstance(loc, int):
-                location_addr += f"[{loc}]"
-            else:
-                location_addr += f"{'.' if i > 0 else ''}{loc}"
+        location_addr: str = "".join(
+            (f"[{loc}]" if isinstance(loc, int) else f"{'.' if i > 0 else ''}{loc}")
+            for i, loc in enumerate(location)
+        )
         headers = ["Type", "Location", "Message"]
         information = [obj_type, location_addr]
 
@@ -109,7 +116,7 @@ def parse_validation_response(
             input_arg = body
             for loc in location:
                 try:
-                    input_arg = input_arg[loc]
+                    input_arg = None if obj_type == "missing" else input_arg[loc]
                 except TypeError:
                     break
             information.append(input_arg)
@@ -124,8 +131,8 @@ def parse_validation_response(
 
 def check_extra(extra_name: str) -> typing.Callable:
     def decorator(
-        class_func: typing.Optional[typing.Callable] = None,
-    ) -> typing.Optional[typing.Callable]:
+        class_func: typing.Callable | None = None,
+    ) -> typing.Callable | None:
         @functools.wraps(class_func)
         def wrapper(self, *args, **kwargs) -> typing.Any:
             if extra_name == "plot" and not all(
@@ -137,10 +144,16 @@ def check_extra(extra_name: str) -> typing.Callable:
                 raise RuntimeError(
                     f"Plotting features require the '{extra_name}' extension to Simvue"
                 )
-            elif extra_name == "torch" and not importlib.util.find_spec("torch"):
-                raise RuntimeError(
-                    "PyTorch features require the 'torch' module to be installed"
-                )
+            elif extra_name == "eco":
+                if not importlib.util.find_spec("geocoder"):
+                    raise RuntimeError(
+                        f"Eco features require the '{extra_name}' extenstion to Simvue"
+                    )
+            elif extra_name == "torch":
+                if not importlib.util.find_spec("torch"):
+                    raise RuntimeError(
+                        "PyTorch features require the 'torch' module to be installed"
+                    )
             elif extra_name not in EXTRAS:
                 raise RuntimeError(f"Unrecognised extra '{extra_name}'")
             return class_func(self, *args, **kwargs) if class_func else None
@@ -150,14 +163,44 @@ def check_extra(extra_name: str) -> typing.Callable:
     return decorator
 
 
-def parse_pydantic_error(class_name: str, error: pydantic.ValidationError) -> str:
+def parse_pydantic_error(error: pydantic.ValidationError) -> str:
     out_table: list[str] = []
     for data in json.loads(error.json()):
+        _input = data.get("input") if data["input"] is not None else "None"
+        if isinstance(_input, dict):
+            _input_str = json.dumps(_input, indent=2)
+            _input_str = "\n".join(
+                f"{line[:47]}..." if len(line) > 50 else line
+                for line in _input_str.split("\n")
+            )
+        else:
+            _input_str = (
+                _input_str
+                if len((_input_str := f"{_input}")) < 50
+                else f"{_input_str[:50]}..."
+            )
+        _type: str = data["type"]
+
+        _skip_type_compare_for = (
+            "error",
+            "missing",
+            "unexpected",
+            "union_tag",
+            "parsing",
+            "scheme",
+            "syntax",
+        )
+
+        if (_input_type := type(_input)) != _type and all(
+            e not in _type for e in _skip_type_compare_for
+        ):
+            _type = f"{_input_type.__name__} != {_type}"
+
         out_table.append(
             [
-                data.get("input") if data["input"] is not None else "None",
+                _input_str,
                 data["loc"],
-                data["type"],
+                _type,
                 data["msg"],
             ]
         )
@@ -166,13 +209,13 @@ def parse_pydantic_error(class_name: str, error: pydantic.ValidationError) -> st
         headers=["Input", "Location", "Type", "Message"],
         tablefmt="fancy_grid",
     )
-    return f"`{class_name}` Validation:\n{err_table}"
+    return f"`{error.title}` Validation:\n{err_table}"
 
 
 def skip_if_failed(
     failure_attr: str,
     ignore_exc_attr: str,
-    on_failure_return: typing.Optional[typing.Any] = None,
+    on_failure_return: typing.Any | None = None,
 ) -> typing.Callable:
     """Decorator for ensuring if Simvue throws an exception any other code continues.
 
@@ -213,7 +256,7 @@ def skip_if_failed(
             try:
                 return class_func(self, *args, **kwargs)
             except pydantic.ValidationError as e:
-                error_str = parse_pydantic_error(class_func.__name__, e)
+                error_str = parse_pydantic_error(e)
                 if getattr(self, ignore_exc_attr, True):
                     setattr(self, failure_attr, True)
                     logger.error(error_str)
@@ -250,7 +293,7 @@ def prettify_pydantic(class_func: typing.Callable) -> typing.Callable:
         try:
             return class_func(self, *args, **kwargs)
         except pydantic.ValidationError as e:
-            error_str = parse_pydantic_error(class_func.__name__, e)
+            error_str = parse_pydantic_error(e)
             raise RuntimeError(error_str)
 
     return wrapper
@@ -278,11 +321,11 @@ def remove_file(filename: str) -> None:
             logger.error("Unable to remove file %s due to: %s", filename, str(err))
 
 
-def get_expiry(token) -> typing.Optional[int]:
+def get_expiry(token) -> int | None:
     """
     Get expiry date from a JWT token
     """
-    expiry: typing.Optional[int] = None
+    expiry: int | None = None
     with contextlib.suppress(jwt.DecodeError):
         expiry = jwt.decode(token, options={"verify_signature": False})["exp"]
 
@@ -301,7 +344,7 @@ def prepare_for_api(data_in, all=True):
     return data
 
 
-def calculate_sha256(filename: str, is_file: bool) -> typing.Optional[str]:
+def calculate_sha256(filename: str | typing.Any, is_file: bool) -> str | None:
     """
     Calculate sha256 checksum of the specified file
     """
@@ -327,36 +370,14 @@ def validate_timestamp(timestamp):
     Validate a user-provided timestamp
     """
     try:
-        datetime.datetime.strptime(timestamp, "%Y-%m-%d %H:%M:%S.%f")
+        datetime.datetime.strptime(timestamp, DATETIME_FORMAT)
     except ValueError:
         return False
 
     return True
 
 
-def compare_alerts(first, second):
-    """ """
-    for key in ("name", "description", "source", "frequency", "notification"):
-        if key in first and key in second:
-            if not first[key]:
-                continue
-
-            if first[key] != second[key]:
-                return False
-
-    if "alerts" in first and "alerts" in second:
-        for key in ("rule", "window", "metric", "threshold", "range_low", "range_high"):
-            if key in first["alerts"] and key in second["alerts"]:
-                if not first[key]:
-                    continue
-
-                if first["alerts"][key] != second["alerts"]["key"]:
-                    return False
-
-    return True
-
-
-def simvue_timestamp(date_time: typing.Optional[datetime.datetime] = None) -> str:
+def simvue_timestamp(date_time: datetime.datetime | None = None) -> str:
     """Return the Simvue valid timestamp
 
     Parameters
@@ -371,4 +392,34 @@ def simvue_timestamp(date_time: typing.Optional[datetime.datetime] = None) -> st
     """
     if not date_time:
         date_time = datetime.datetime.now(timezone.utc)
-    return date_time.strftime("%Y-%m-%d %H:%M:%S.%f")
+    return date_time.strftime(DATETIME_FORMAT)
+
+
+@functools.lru_cache
+def get_mimetypes() -> list[str]:
+    """Returns a list of allowed MIME types"""
+    mimetypes.init()
+    _valid_mimetypes = ["application/vnd.plotly.v1+json"]
+    _valid_mimetypes += list(mimetypes.types_map.values())
+    return _valid_mimetypes
+
+
+def get_mimetype_for_file(file_path: pathlib.Path) -> str:
+    """Return MIME type for the given file"""
+    _guess, *_ = mimetypes.guess_type(file_path)
+    return _guess or "application/octet-stream"
+
+
+# Create a new Merge strategy for merging local file and staging attributes
+staging_merger = Merger(
+    # pass in a list of tuple, with the
+    # strategies you are looking to apply
+    # to each type.
+    [(list, ["override"]), (dict, ["merge"]), (set, ["union"])],
+    # next, choose the fallback strategies,
+    # applied to all other types:
+    ["override"],
+    # finally, choose the strategies in
+    # the case where the types conflict:
+    ["override"],
+)
